using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ICPDAS;

namespace DO_WriteValue
{
    public partial class FormDO_WriteValue : Form
    {
        static readonly ushort[] USBIO_DID = new ushort[] { ICPDAS_USBIO.USB2026, ICPDAS_USBIO.USB2045, ICPDAS_USBIO.USB2055, ICPDAS_USBIO.USB2064, ICPDAS_USBIO.USB2045_32, ICPDAS_USBIO.USB2055_32, ICPDAS_USBIO.USB2064_16, ICPDAS_USBIO.USB2068_18 };
        static readonly UInt32 COMM_TIMEOUT = 500;
        static readonly ushort USBIO_2055_32 =  ICPDAS_USBIO.USB2055_32;

        private ICPDAS_USBIO m_USBIO;
        private byte m_byDITotal;
        private byte m_byDOTotal;
        private byte m_byAITotal;
        private byte m_byAOTotal;
        private byte m_byPITotal;
        private byte m_byPOTotal;

        private byte Asc2Hex(byte i_byAscCode)
        {
            if (i_byAscCode >= '0' && i_byAscCode <= '9')
                return (byte)(i_byAscCode - '0');
            else if (i_byAscCode >= 'a' && i_byAscCode <= 'f')
                return (byte)(i_byAscCode - 'a' + 10);
            else if (i_byAscCode >= 'A' && i_byAscCode <= 'F')
                return (byte)(i_byAscCode - 'A' + 10);
            else
                return 0;
        }

        public FormDO_WriteValue()
        {
            InitializeComponent();

            cbxDevID.SelectedIndex = 0;

            m_USBIO = new ICPDAS_USBIO();
        }

        private void btnOpen_Click(object sender, EventArgs e)
        {
            int iErrCode;

            m_USBIO.SetCommTimeout(COMM_TIMEOUT);

            if ((int)ICPDAS_USBIO.ERR_CODE.ERR_NO_ERR != (iErrCode = m_USBIO.OpenDevice(USBIO_2055_32,1)))
                MessageBox.Show("Failed to open device. ErrCode:[" + iErrCode.ToString() + "]");
            else
            {
                UInt32 dwSoftWDTTimeout;
                UInt16 wFWVer;
                byte[] byDeviceSN;
                byte[] byDeviceNickName;
                byte bySupIOMask;

                m_USBIO.GetSoftWDTTimeout(out dwSoftWDTTimeout);
                lvInfo.Items.Add("SoftWDT: " + dwSoftWDTTimeout.ToString());

                m_USBIO.GetFwVer(out wFWVer);
                lvInfo.Items.Add("FW ver: " + wFWVer.ToString());

                m_USBIO.GetDeviceNickName(out byDeviceNickName);
                lvInfo.Items.Add("Description: " + Encoding.ASCII.GetString(byDeviceNickName));

                m_USBIO.GetDeviceSN(out byDeviceSN);
                lvInfo.Items.Add("SN: " + Encoding.ASCII.GetString(byDeviceSN));

                m_USBIO.GetSupportIOMask(out bySupIOMask);
                lvInfo.Items.Add("Support IO Mask: 0x" + bySupIOMask.ToString("X02"));

                if (ICPDAS_USBIO.SUPPORT_DI(bySupIOMask))
                {
                    m_USBIO.GetDITotal(out m_byDITotal);
                    lvInfo.Items.Add("DI channels: " + m_byDITotal.ToString());
                }

                if (ICPDAS_USBIO.SUPPORT_DO(bySupIOMask))
                {
                    m_USBIO.GetDOTotal(out m_byDOTotal);
                    lvInfo.Items.Add("DO channels: " + m_byDOTotal.ToString());
                }

                if (ICPDAS_USBIO.SUPPORT_AI(bySupIOMask))
                {
                    m_USBIO.GetAITotal(out m_byAITotal);
                    lvInfo.Items.Add("AI channels: " + m_byAITotal.ToString());
                }

                if (ICPDAS_USBIO.SUPPORT_AO(bySupIOMask))
                {
                    m_USBIO.GetAOTotal(out m_byAOTotal);
                    lvInfo.Items.Add("AO channels: " + m_byAOTotal.ToString());
                }

                if (ICPDAS_USBIO.SUPPORT_PI(bySupIOMask))
                {
                    m_USBIO.GetPITotal(out m_byPITotal);
                    lvInfo.Items.Add("PI channels: " + m_byPITotal.ToString());
                }

                if (ICPDAS_USBIO.SUPPORT_PO(bySupIOMask))
                {
                    m_USBIO.GetPOTotal(out m_byPOTotal);
                    lvInfo.Items.Add("PO channels: " + m_byPOTotal.ToString());
                }

                txtWriteDOValue.Enabled = true;
                btnWriteDOValue.Enabled = true;
                btnReadDOValue.Enabled = true;
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            int iErrCode;

            lvInfo.Items.Clear();
            txtReadDOValue.Text = "";
            txtWriteDOValue.Text = "";

            if ((int)ICPDAS_USBIO.ERR_CODE.ERR_NO_ERR != (iErrCode = m_USBIO.CloseDevice()))
                MessageBox.Show("Failed to close device. ErrCode:[" + iErrCode.ToString() + "]");

            txtWriteDOValue.Enabled = false;
            btnWriteDOValue.Enabled = false;
            btnReadDOValue.Enabled = false;
        }

        private void btnReadDOValue_Click(object sender, EventArgs e)
        {
            int iErrCode;
            byte[] byDOValue = new byte[(m_byDOTotal + 7) / 8];
            
            if ((int)ICPDAS_USBIO.ERR_CODE.ERR_NO_ERR == (iErrCode = m_USBIO.DO_ReadValue(byDOValue)))
            {
                txtReadDOValue.Text = "";

                for (int iIdx = 0; iIdx < ((m_byDOTotal + 7) / 8); iIdx++)
                    txtReadDOValue.Text = byDOValue[iIdx].ToString("X02") + txtReadDOValue.Text;
            }
            else
                MessageBox.Show("Failed to read DO value. ErrCode:[" + iErrCode.ToString() + "]");
        }

        private void btnWriteDOValue_Click(object sender, EventArgs e)
        {
            int iErrCode;
            byte[] byDOValueStringArray;
            byte[] byDOValue = new byte[(m_byDOTotal + 7) / 8];

            byDOValueStringArray = ASCIIEncoding.ASCII.GetBytes(txtWriteDOValue.Text);
            if (byDOValueStringArray.Length > (((m_byDOTotal + 7) / 8) * 2))
            {
                MessageBox.Show("DO output invalid. (Do output over channel number)");
                return;
            }
            Array.Clear(byDOValue, 0, byDOValue.Length);
            for (int iIdx = 0; iIdx < byDOValueStringArray.Length; iIdx++)
                byDOValue[(byDOValueStringArray.Length - 1 - iIdx) / 2] += Convert.ToByte(Asc2Hex(byDOValueStringArray[iIdx]) * (Math.Pow(16, (byDOValueStringArray.Length - 1 - iIdx) % 2)));

            if ((int)ICPDAS_USBIO.ERR_CODE.ERR_NO_ERR != (iErrCode = m_USBIO.DO_WriteValue(byDOValue)))
                MessageBox.Show("Failed to write DO value. ErrCode:[" + iErrCode.ToString() + "]");
            else
                MessageBox.Show("Success.");
        }
    }
}